﻿using System.Collections.Specialized;
using System.Windows;
using FlatDataSourceBasic.WPF.Data;
using Infragistics.Controls.Grids;
using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;

namespace FlatDataSourceBasic.WPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private ExecutionContext? _expandedContext;

        public MainWindow()
        {
            InitializeComponent();

            FlatDataSource flatDataSource = new FlatDataSource();

            // expand by default the result as far as filter tree is expanded
            flatDataSource.HierarchyExpansionMode = HierarchyExpansionMode.DrillDown;

            // listening for new item is added in order to expand the root node
            // when working with flat data this is the most easiest way to do that 
            flatDataSource.Columns.CollectionChanging += this.ColumnsAreaItems_CollectionChanging;
            flatDataSource.Columns.CollectionChanged += this.ColumnsAreaItems_CollectionChanged;

            flatDataSource.Rows.CollectionChanging += this.RowsAreaItems_CollectionChanging;
            flatDataSource.Rows.CollectionChanged += this.RowsAreaItems_CollectionChanged;

            this.SetInitialItems(flatDataSource);
            this.SetHierarchies(flatDataSource);
            this.SetMetadata(flatDataSource);

            flatDataSource.ResultChanged += this.FlatDataSource_ResultChanged;
            flatDataSource.ItemsSource = ProductDataGenerator.Generate();

            xamPivotGrid.AxisExpansionChanging += this.PivotGrid_AxisExpansionChanging;
            xamPivotGrid.DataSource = flatDataSource;
            xamPivotDataSelector.DataSource = flatDataSource;
        }

        private void RowsAreaItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            FlatDataSource flatDataSource = (FlatDataSource)xamPivotGrid.DataSource;
            flatDataSource.Rows.CollectionChanging -= this.RowsAreaItems_CollectionChanging;
            flatDataSource.Rows.CollectionChanged -= this.RowsAreaItems_CollectionChanged;
        }

        private void ColumnsAreaItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            FlatDataSource flatDataSource = (FlatDataSource)xamPivotGrid.DataSource;
            flatDataSource.Columns.CollectionChanging -= this.ColumnsAreaItems_CollectionChanging;
            flatDataSource.Columns.CollectionChanged -= this.ColumnsAreaItems_CollectionChanged;
        }

        private void PivotGrid_AxisExpansionChanging(object sender, Infragistics.Controls.Grids.PivotCancellableAxisEventArgs e)
        {
            // cache the expansion context
            if (e.HeaderCell is PivotRowHeaderCell)
            {
                this._expandedContext = ExecutionContext.Rows;
            }
            else
            {
                this._expandedContext = ExecutionContext.Columns;
            }
        }

        private void FlatDataSource_ResultChanged(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
        {
            if (this._expandedContext.HasValue)
            {
                FlatDataSource flatDataSource = (FlatDataSource)sender;

                // put it back to DrillDown because of the bug workaround at line: 149
                flatDataSource.HierarchyExpansionMode = HierarchyExpansionMode.DrillDown;                
                flatDataSource.UpdateVisualMembers(this._expandedContext.Value);

                this._expandedContext = null;
                flatDataSource.RefreshGrid();
            }
        }

        private void RowsAreaItems_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (var newItem in e.NewItems)
                {
                    IFilterViewModel filterViewModel = newItem as IFilterViewModel;
                    if (filterViewModel != null && filterViewModel.FilterMembers != null)
                    {
                        // expand the root level members
                        foreach (IFilterMember filterMember in filterViewModel.FilterMembers)
                        {
                            // on flat data source this will immediately load the children members
                            filterMember.IsExpanded = true;
                            foreach (IFilterMember member in filterMember.FilterMembers)
                            {
                                // listen for when this member IsExpanded is changed
                                member.PropertyChanged += this.FilterMember_PropertyChanged;
                            }
                        }
                    }
                }
            }
        }

        private void ColumnsAreaItems_CollectionChanging(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (var newItem in e.NewItems)
                {
                    IFilterViewModel filterViewModel = newItem as IFilterViewModel;
                    if (filterViewModel != null && filterViewModel.FilterMembers != null)
                    {
                        // expand the root level members
                        foreach (IFilterMember filterMember in filterViewModel.FilterMembers)
                        {
                            // on flat data source this will immediately load the children members
                            filterMember.IsExpanded = true;
                            this.DrillDownMember(filterMember);
                        }
                    }
                }
            }
        }

        private void FilterMember_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            // expand the whole sub hierarchy
            if (e.PropertyName == "IsExpanded")
            {
                IFilterMember filterMember = (IFilterMember)sender;
                if (!filterMember.IsExpanded)
                {
                    // clear the context since we are collapsing the hierarchy                 
                    this._expandedContext = null;
                    return;
                }

                // for now put this to SinlgeLevelOnly because of a bug; it will be fixed in the upcoming SR
                ((DataSourceBase)xamPivotGrid.DataSource).HierarchyExpansionMode = HierarchyExpansionMode.SingleLevelOnly;
                if (this._expandedContext.HasValue)
                {
                    this.DrillDownMember(filterMember);
                }
            }
        }

        private void DrillDownMember(IFilterMember filterMember)
        {
            if (filterMember.IsExpanded && filterMember.FilterMembers != null)
            {
                foreach (IFilterMember member in filterMember.FilterMembers)
                {
                    member.IsExpanded = true;
                    this.DrillDownMember(member);
                }
            }
        }

        private void SetInitialItems(FlatDataSource flatDataSource)
        {
            flatDataSource.Rows = DataSourceBase.GenerateInitialItems("[ProductCategory].[Product Categories]");
            flatDataSource.Columns = DataSourceBase.GenerateInitialItems("[Region].[Destributors by Regions]");
            flatDataSource.Measures = DataSourceBase.GenerateInitialItems("Cost");
        }

        private void SetHierarchies(FlatDataSource flatDataSource)
        {
            // ##### [ProductCategory].[Product Categories] Hierarchy #####
            // suppose we don't have the generic type parameter (dynamic type)
            HierarchyDescriptor productHierarchy =
                new HierarchyDescriptor
                {
                    SourcePropertyName = "ProductCategory",
                    HierarchyName = "Product Categories"
                };

            // use this method to add the root level
            productHierarchy.AddLevel("All Products", "All Products");

            HierarchyLevelDescriptor categoriesLevel = new HierarchyLevelDescriptor();
            categoriesLevel.LevelExpressionPath = "ProductCategory";
            categoriesLevel.LevelName = "Categories";

            HierarchyLevelDescriptor brandsLevel = new HierarchyLevelDescriptor();
            brandsLevel.LevelExpressionPath = "Brand";
            brandsLevel.LevelName = "Brands";

            HierarchyLevelDescriptor productsLevel = new HierarchyLevelDescriptor();
            productsLevel.LevelExpressionPath = "ProductName";
            productsLevel.LevelName = "Products";

            // add the levels to the hierarchy
            productHierarchy.LevelDescriptors.Add(categoriesLevel);
            productHierarchy.LevelDescriptors.Add(brandsLevel);
            productHierarchy.LevelDescriptors.Add(productsLevel);

            flatDataSource.AddHierarchyDescriptor(productHierarchy);

            // ##### [Region].[Destributors by Regions] Hierarchy #####
            HierarchyDescriptor regionHierarchy =
                new HierarchyDescriptor()
                {
                    SourcePropertyName = "Region",
                    HierarchyName = "Destributors by Regions"
                };

            // use this method to add the root level
            regionHierarchy.AddLevel("All Regions", "All Regions");

            HierarchyLevelDescriptor regionLevel = new HierarchyLevelDescriptor();
            regionLevel.LevelExpressionPath = "Region";
            regionLevel.LevelName = "Regions";

            HierarchyLevelDescriptor distributorsLevel = new HierarchyLevelDescriptor();
            distributorsLevel.LevelExpressionPath = "Distributor";
            distributorsLevel.LevelName = "Distributors";

            // add the levels to the hierarchy
            regionHierarchy.LevelDescriptors.Add(regionLevel);
            regionHierarchy.LevelDescriptors.Add(distributorsLevel);

            flatDataSource.AddHierarchyDescriptor(regionHierarchy);
        }

        private void SetMetadata(FlatDataSource flatDataSource)
        {
            CubeMetadata cubeMetadata =
                new CubeMetadata { DataTypeFullName = typeof(ProductData).FullName, DisplayName = "Products Data" };
            flatDataSource.CubesSettings.Add(cubeMetadata);

            DimensionMetadata costMetadata = new DimensionMetadata();
            costMetadata.SourcePropertyName = "Cost";
            costMetadata.DimensionType = DimensionType.Measure;

            cubeMetadata.DimensionSettings.Add(costMetadata);
        }
    }
}
